unit ME1400Klasse;

interface




Uses Windows, SysUtils, Dialogs, Forms, mmSystem, CPort;





Type tcommand = (coStandby, coLeerlauf, coChromatogramm, coSpektrum,
                 coParameter, coSpektrTyp);

     tstatus  = (stBereit, stNichtBereit);

     errorstring = array [0..255] of char;

     //Funktionstypen der allgemeine Routinen zur Me1400-Programmierung

     TGetDllVersion  = Function : Integer; cdecl;

     TGetDrvErrMess  = Function(Var cErrortext : errorstring) : Integer; cdecl;

     TEnableInt      = Function(iBoardNumber : integer;
                                IrqFunc      : Pointer;
                                Nummer       : integer) : Integer; cdecl;

     TDisableInt     = Function(iBoardNumber, Nummer: integer) : integer; cdecl;

     TBoardVersion   = Function(    iBoardNumber : Integer;
                                Var iDevices     : Integer): Integer; cdecl;

     TGetIqrCnt      = Function(    iBoardNumber : integer;
                                var iIrqCnt      : integer) : Integer; cdecl;

     //TgetDevInfo   = Function(    iBoardNumber : integer;
     //                         var DevInfo      : DEVICEINFOSTRUCT)
     //                : Integer; cdecl;

     TSetMultifunctionPin = Function(iBoardNumber : integer;
                                     MultiPin     : integer) : Integer; cdecl;

     //Funktionstypen zur Programmierung des 8254-Counters

     TInitModeTimerA = Function(iBoardNumber : integer;
                                CtrlWordA    : integer) : Integer; cdecl;

     TInitModeTimerB = Function(iBoardNumber : integer;
                                CtrlWordB    : integer) : Integer; cdecl;

     TCntWrite       = Function(    BoardNumber,
                                    iCounterNo,
                                    iMode,
                                    iValue       : integer) : integer; cdecl;
     TCntRead        = Function(    BoardNumber,
                                    iCounterNo   : integer;
                                var iValue       : integer) : integer; cdecl;

     //Funktionstypen fr den Digital-I/O-Chip

     TDIOSetPortDirection = Function(iBoardNumber,
                                     iPortNo,
                                     iDir          : integer): integer; cdecl;

     TDIGetBit            = Function(    iBoardNumber,
                                         iPortNo,
                                         iBitNo     : integer;
                                     var iBitValue  : integer) : integer; cdecl;

     TDIGetByte           = Function(    iBoardNumber,
                                         iPortNo    : integer;
                                     var iValue     : integer) : integer; cdecl;

     TDOSetBit            = Function(iBoardNumber,
                                     iPortNo,
                                     iBitNo,
                                     iBitValue      : integer) : integer; cdecl;

     TDOSetByte           = Function(iBoardNumber,
                                     iPortNo,
                                     iValue         : integer) : integer; cdecl;

     //Objekt zur ansteuereung der Me1400-Karte

     TMe1400 = Class (TObject)

               Private

                 DLLHandle            : THandle; //Der Dll-Handle

                 dllvorhanden_b       : Boolean; //Gibt es die Dll mit den
                                                 //Funktionen berhaupt?

                 //Die Zeiger auf die zur Verfgung stehenden allgemeinen
                 //Funktionen der Me1400
                 GetDllVersionFarProc       : TFarProc;
                 GetDrvErrMessFarProc       : TFarProc;
                 EnableIntFarProc           : TFarProc;
                 DisableIntFarProc          : TFarProc;
                 BoardVersionFarProc        : TFarProc;
                 GetIqrCntFarProc           : TFarProc;
                 SetMultifunctionPinFarProc : TFarProc;

                 //Die Zeiger auf die Timer-Counter-Funktionen
                 InitModeTimerAFarProc      : TFarProc;
                 InitModeTimerBFarProc      : TFarProc;
                 CntWriteFarProc            : TFarProc;
                 CntReadFarProc             : TFarProc;

                 //Die Zeiger auf die Digital-I/O-Funktionen
                 DIOSetPortDirectionFarProc : TFarProc;
                 DIGetBitFarProc            : TFarProc;
                 DIGetByteFarProc           : TFarProc;
                 DOSetBitFarProc            : TFarProc;
                 DOSetByteFarProc           : TFarProc;

                 //Die zur Verfgung stehenden allgemeinen
                 //Funktionen der Me1400
                 GetDllVersion       : TGetDllVersion;
                 GetDrvErrMess       : TGetDrvErrMess;
                 EnableInt           : TEnableInt;
                 DisableInt          : TDisableInt;
                 BoardVersion        : TBoardVersion;
                 GetIqrCnt           : TGetIqrCnt;
                 SetMultifunctionPin : TSetMultifunctionPin;

                 //Die Timer-Counter-Funktionen
                 InitModeTimerA      : TInitModeTimerA;
                 InitModeTimerB      : TInitModeTimerB;
                 CntWrite            : TCntWrite;
                 CntRead             : TCntRead;

                 //Die Digital-I/O-Funktionen
                 DIOSetPortDirection : TDIOSetPortDirection;
                 DIGetBit            : TDIGetBit;
                 DIGetByte           : TDIGetByte;
                 DOSetBit            : TDOSetBit;
                 DOSetByte           : TDOSetByte;

                 //Variablen zur Prfung, ob Spektrograph nicht schon gestartet
                 //ist und die Timer-Multi-IO-Karte blockiert
                 MutexHandle      : THandle;
                 Wnd              : HWnd;
                 schongestartet_b : Boolean;

               Public

                 Constructor Create;
                 Destructor Destroy; Override;

                 Procedure io_richtung_setzen; //Setz Port A auf Senden und
                                               //Port B auf empfangen.

                 Procedure belichtungszeit_setzen(blz_i : Integer);
                           //bergibt die Belichtungszeit in Millisekunden
                           //an das Timer-Counter-System

                 Function SendCommand_b(    command : tcommand;
                                         Var ComPort : TComPort) : Boolean;

                 Procedure InStandbySetzen;

                 Function bereit_b : Boolean; //Gibt aus, ob das Spektrometer
                                              //in der Lage ist, einen Triger
                                              //zum Start/beenden einer Messung
                                              //entgegenzunehmen.

                 Property timer_io_ok_b : Boolean Read dllvorhanden_b;

                 Property SchonGestartet_b_Prop : Boolean Read schongestartet_b;

               End;





implementation




Var ZeitVorbei_b : Boolean;




Procedure TimeCallback(TimerID, MSG     : UINT;
                       dwUser, dw1, dw2 : DWord); Pascal;
Begin
  zeitvorbei_b := True;
End;




Constructor TMe1400.Create;

Var windir_s    : String;
    PufferPChar : PChar;

Begin
  //Vor dem Versuch die DLL zu laden wird festgestellt, ob Spektrograph schon
  //gestartet worden ist. Die Meilhaus-Timer-IO-Karte darf nmlich
  //nicht von zwei Instanzen von Spektrograph gleichzeitig benutzt werden, da
  //dies zu Konflikten bei der Spektrometersteuerung, wenn nicht sogar zum
  //Absturz fhrt.
  schongestartet_b := False;
  MutexHandle := CreateMutex(Nil, True, 'Doppelstart');
  If GetLastError = ERROR_ALREADY_EXISTS Then
     Begin
       Wnd := 0;
       Repeat
         Wnd := FindWindowEx(0, Wnd,'TApplication', PChar(Application.Title));
       Until Not(Wnd = Application.Handle);
       If Not(Wnd = 0) Then schongestartet_b := True;
     End;
  ReleaseMutex(MutexHandle);
  If schongestartet_b Then
     Begin
       dllvorhanden_b := False;
       Exit
     End;

  //Dll einbinden
  PufferPChar := StrAlloc(Succ(MAX_PATH));
  GetSystemDirectory(PufferPChar, Succ(MAX_PATH));
  windir_s := PufferPChar;
  StrDispose(PufferPChar);
  If FileExists(windir_s + '\me1400.Dll') Then
     DLLHandle := LoadLibrary(PChar('me1400.Dll'))
     Else
     Begin
       dllvorhanden_b := False;
       Exit
     End;

  //Zeiger auf die einzelnen Funktionen der Dll ermitteln:
  GetDllVersionFarProc := GetProcAddress(DLLHandle, '_me1400GetDLLVersion');
  GetDrvErrMessFarProc := GetProcAddress(DllHandle, '_me1400GetDrvErrMess');
  EnableIntFarProc     := GetProcAddress(DllHandle, '_me1400EnableInt');
  DisableIntFarProc    := GetProcAddress(DllHandle, '_me1400DisableInt');
  BoardVersionFarProc  := GetProcAddress(DLLHandle, '_me1400BoardVersion');
  GetIqrCntFarProc     := GetProcAddress(DllHandle, '_me1400GetIrqCnt');
  SetMultifunctionPinFarProc
                       := GetProcAddress(DLLHandle,
                                                  '_me1400SetMultifunctionPin');

  InitModeTimerAFarProc := GetProcAddress(DllHandle, '_me1400InitModeTimerA');
  InitModeTimerBFarProc := GetProcAddress(DllHandle, '_me1400InitModeTimerB');
  CntWriteFarProc       := GetProcAddress(DLLHandle, '_me1400CntWrite');
  CntReadFarProc        := GetProcAddress(DLLHandle, '_me1400CntRead');

  DIOSetPortDirectionFarProc
                   := GetProcAddress(DllHandle, '_me1400DIOSetPortDirection');
  DIGetBitFarProc  := GetProcAddress(DllHandle, '_me1400DIGetBit');
  DIGetByteFarProc := GetProcAddress(DllHandle, '_me1400DIGetByte');
  DOSetBitFarProc  := GetProcAddress(DllHandle, '_me1400DOSetBit');
  DOSetByteFarProc := GetProcAddress(DllHandle, '_me1400DOSetByte');

  //Nachschauen, ob auch alle Zeiger ermittlet worden sind.
  If Not(GetDllVersionFarProc       = Nil) And
     Not(GetDrvErrMessFarProc       = Nil) And
     Not(EnableIntFarProc           = Nil) And
     Not(DisableIntFarProc          = Nil) And
     Not(BoardVersionFarProc        = Nil) And
     Not(GetIqrCntFarProc           = Nil) And
     Not(SetMultifunctionPinFarProc = Nil) And
     Not(InitModeTimerAFarProc      = Nil) And
     Not(InitModeTimerBFarProc      = Nil) And
     Not(CntWriteFarProc            = Nil) And
     Not(CntReadFarProc             = Nil) And
     Not(DIOSetPortDirectionFarProc = Nil) And
     Not(DIGetBitFarProc            = Nil) And
     Not(DIGetByteFarProc           = Nil) And
     Not(DOSetBitFarProc            = Nil) And
     Not(DOSetByteFarProc           = Nil) Then
     Begin
       //Falls Ja, den Zeiger an die Funktions schnittstelle bergeben.
       @GetDllVersion      := GetDllVersionFarProc;
       @GetDrvErrMess       := GetDrvErrMessFarProc;
       @EnableInt           := EnableIntFarProc;
       @DisableInt          := DisableIntFarProc;
       @BoardVersion        := BoardVersionFarProc;
       @GetIqrCnt           := GetIqrCntFarProc;
       @SetMultifunctionPin := SetMultifunctionPinFarProc;

       @InitModeTimerA := InitModeTimerAFarProc;
       @InitModeTimerB := InitModeTimerBFarProc;
       @CntWrite       := CntWriteFarProc;
       @CntRead        := CntReadFarProc;

       @DIOSetPortDirection := DIOSetPortDirectionFarProc;
       @DIGetBit            := DIGetBitFarProc;
       @DIGetByte           := DIGetByteFarProc;
       @DOSetBit            := DOSetBitFarProc;
       @DOSetByte           := DOSetByteFarProc;

       dllvorhanden_b := True; //Falls ja, die Variable dllvorhanden_b setzen


     End
     Else dllvorhanden_b := False; //Falls nein, die Variable dllvorhanden auf
                                   //false setzen um zu signalisieren, dass
                                   //die Dll nicht vorhanden oder nicht
                                   //funktionstchtig ist.
End;



Destructor TMe1400.Destroy;
Begin
  //Die Recorsen fr alle Funktionen freigeben
  GetDllVersionFarProc       := Nil;
  GetDrvErrMessFarProc       := Nil;
  EnableIntFarProc           := Nil;
  DisableIntFarProc          := Nil;
  GetIqrCntFarProc           := Nil;
  BoardVersionFarProc        := Nil;
  SetMultifunctionPinFarProc := Nil;

  InitModeTimerAFarProc := Nil;
  InitModeTimerBFarProc := Nil;
  CntWriteFarProc       := Nil;
  CntReadFarProc        := Nil;

  DIOSetPortDirectionFarProc := Nil;
  DIGetBitFarProc            := Nil;
  DIGetByteFarProc           := Nil;
  DOSetBitFarProc            := Nil;
  DOSetByteFarProc           := Nil;

  FreeLibrary(DLLHandle); //Dll freigeben

  Inherited Destroy
End;



Procedure TMe1400.belichtungszeit_setzen(blz_i : Integer);
//Die Belichtungszeit bzl_i in Millisekunden wird dem Timer-Counter-System
//bermittelt: alle blz_i-Millisekunden wechselt der TTL-Pegel von Counter 1
Begin
  InitModeTimerA(0, $08 Or $04 Or $02 Or $00);
  //Timer 0 und 1 kaskadiert im Modus 3
  CntWrite(0, 0, 3, 200);
  CntWrite(0, 1, 3, blz_i * 100);
End;



Procedure TMe1400.io_richtung_setzen;
Begin
  DIOSetPortDirection(0, $00, $01);
  DIOSetPortDirection(0, $01, $00)
End;



Function TMe1400.SendCommand_b(    command : tcommand;
                               Var ComPort : TComPort) : Boolean;

Var i            : Integer;
    Commando_by  : Byte;
    SpektrTyp_by : Byte;

Begin
  //Schnittstelle ffnen und zurcksetzen
  Try ComPort.Open;
      Result := True;
  Except Result := False;
  End;
  If Result Then
     Begin
       Try ComPort.ClearBuffer(TRue, True);
       Except ComPort.Close;
              Result := False;
       End
     End;

  //Kommando an das Spektrometer weiterleiten und Besttigung ber die RS232
  //empfangen
  If Result Then
     Begin
       DOSetByte(0, $00, Ord(Command));
       If Not(command = coStandby) Then
          For i := 1 To 3 Do
              If Result Then
                 If (ComPort.Read(commando_by, 1) = 0) Or
                    Not(commando_by = Ord(Command))    Then Result := False;
     End;

  //Schlieen der seriellen Schnittstelle, falls keine weiteren Daten erwartet
  //werden
  If (Command = coLeerlauf) Or (Command = coStandby) Then
     Try ComPort.Close;
     Except Result := False
     End;

  //Falls ein Fehler auftritt, wird das Spektrometer in den Leerlauf versetzt
  If Not(Result) Then DOSetByte(0, $00, Ord(coLeerlauf));
End;



Procedure TMe1400.InStandbySetzen;
Begin
  DIOSetPortDirection(0, $00, $01);
  DOSetByte(0, $00, Ord(coStandby))
End;



Function TMe1400.bereit_b : Boolean;

var puffer_i : Integer;

Begin
  DIGetByte(0, $01, puffer_i);
  If puffer_i = Ord(stBereit) Then Result := True
     Else Result := False
End;




Initialization





Finalization





end.
