unit StringArrayKlasse;

interface


Type

TStringArray = Class (TObject)

  Private

  Zellen_da_s   : Array Of Array Of String;
  zeilenzahl_i  : Integer;
  spaltenzahl_i : Integer;
  nachkomma_i   : Integer;
  error_b       : Boolean;
  Function ResetError_b : Boolean;
  Procedure nachkommasetzen(kommastellen_i : Integer);

  Public

  Constructor Create(GSpaltenzahl_i,
                     GZeilenzahl_i,
                     GNachkommastellen_i : Integer);
  Destructor Destroy; Override;
  Function StrRead_s(Spalte_i, Zeile_i : Integer) : String;
  Procedure StrWrite(Spalte_i, Zeile_i : Integer; was_s : String);
  Function FloatRead_e(Spalte_i, Zeile_i : Integer) : Extended;
  Procedure FloatWrite(Spalte_i, Zeile_i : Integer; was_e : Extended);
  Procedure FloatVonBisWrite(Spalte_i, Zeile_i : Integer;
                             von_e, bis_e      : Extended);
  Function IntRead_i(Spalte_i, Zeile_i : Integer) : Integer;
  Procedure IntWrite(Spalte_i, Zeile_i : Integer; was_i : Integer);
  Function ZeileAuslesen_s(Zeile_i, von_i, bis_i : Integer;
                            Trennzeichen_c        : Char) : String;
  Procedure zeile_loeschen(Zeile_i : Integer);
  Function ZeilenFuellstatus_i(Spalte_i : Integer) : Integer;
  Function SpaltenFuellstatus_i(Zeile_i : Integer) : Integer;
  Function voll_b(Spalte_i : Integer) : Boolean;
  Procedure leeren;

  Property fehler_b_prop    : Boolean Read ResetError_b;
  Property zeilen_i_prop    : Integer Read zeilenzahl_i;
  Property spalten_i_prop   : Integer Read spaltenzahl_i;
  Property nachkomma_i_prop : Integer Read nachkomma_i Write nachkommasetzen;
End;


implementation


Uses sysutils, math;


Constructor TStringArray.Create(GSpaltenzahl_i,
                                GZeilenzahl_i,
                                GNachkommastellen_i  : Integer);

Var z, s : Integer;

Begin
  error_b := False;
  SetLength(Zellen_da_s, Succ(GSpaltenzahl_i), Succ(GZeilenzahl_i));
  zeilenzahl_i  := GZeilenzahl_i;
  spaltenzahl_i := GSpaltenzahl_i;
  nachkommasetzen(GNachkommastellen_i)
End;


Destructor TStringArray.Destroy;

Var i : Integer;

Begin
  For i := 0 To High(zellen_da_s) Do Zellen_da_s[i] := Nil;
  Zellen_da_s := Nil;
  Inherited Destroy;
End;


Function TStringArray.ResetError_b : Boolean;
Begin
  Result  := error_b;
  error_b := False
End;


Procedure TStringArray.nachkommasetzen(kommastellen_i : Integer);
Begin
  If (kommastellen_i > 0) And (kommastellen_i < 19) Then
     nachkomma_i := kommastellen_i
     Else error_b := True
End;


Function TStringArray.StrRead_s(Spalte_i, Zeile_i : Integer) : String;
Begin
  If (Spalte_i >  -1)            And (Zeile_i >  -1)           And
     (Spalte_i <= Spaltenzahl_i) And (Zeile_i <= Zeilenzahl_i) Then
     Result := Zellen_da_s[Spalte_i, Zeile_i]
     Else
     Begin
       Result  := '';
       error_b := True
     End;
End;


Procedure TStringArray.StrWrite(Spalte_i, Zeile_i : Integer; was_s : String);
Begin
  If (Spalte_i >  -1)            And (Zeile_i >  -1)           And
     (Spalte_i <= Spaltenzahl_i) And (Zeile_i <= Zeilenzahl_i) Then
     Zellen_da_s[Spalte_i, Zeile_i] := was_s
     Else error_b := True
End;


Function TStringArray.IntRead_i(Spalte_i, Zeile_i : Integer) : Integer;
Begin
  Try If (Spalte_i >  -1)            And (Zeile_i >  -1)           And
         (Spalte_i <= Spaltenzahl_i) And (Zeile_i <= Zeilenzahl_i) Then
         Result := StrToInt(Zellen_da_s[Spalte_i, Zeile_i])
         Else error_b := True;
  Except error_b := True
  End
End;


Procedure TStringArray.IntWrite(Spalte_i, Zeile_i : Integer;
                                was_i             : Integer);
Begin
  Try If (Spalte_i >  -1)            And (Zeile_i >  -1)           And
         (Spalte_i <= Spaltenzahl_i) And (Zeile_i <= Zeilenzahl_i) Then
         Zellen_da_s[Spalte_i, Zeile_i] := IntToStr(was_i)
         Else error_b := True;
  Except error_b := True
  End
End;


Function TStringArray.FloatRead_e(Spalte_i, Zeile_i : Integer) : Extended;
Begin
  Try If (Spalte_i >  -1)            And (Zeile_i >  -1)           And
         (Spalte_i <= Spaltenzahl_i) And (Zeile_i <= Zeilenzahl_i) Then
         Result := StrToFloat(Zellen_da_s[Spalte_i, Zeile_i])
         Else error_b := True;
  Except error_b := True
  End
End;


Procedure TStringArray.FloatWrite(Spalte_i, Zeile_i : Integer;
                                  was_e             : Extended);
Begin
  Try If (Spalte_i >  -1)            And (Zeile_i >  -1)           And
         (Spalte_i <= Spaltenzahl_i) And (Zeile_i <= Zeilenzahl_i) Then
         Zellen_da_s[Spalte_i, Zeile_i]
                           :=   FloatToStr(Round(was_e * Power(10, nachkomma_i))
                              / Power(10, nachkomma_i))
         Else error_b := True;
  Except error_b := True
  End
End;


Procedure TStringArray.FloatVonBisWrite(Spalte_i, Zeile_i : Integer;
                                        von_e, bis_e      : Extended);
Begin
  Try If (Spalte_i >  -1)            And (Zeile_i >  -1)           And
         (Spalte_i <= Spaltenzahl_i) And (Zeile_i <= Zeilenzahl_i) Then
         Zellen_da_s[Spalte_i, Zeile_i]
                           :=     FloatToStr(Round(von_e*Power(10, nachkomma_i))
                                / Power(10, nachkomma_i))
                              + ' - '
                              +   FloatToStr(Round(bis_e*Power(10, nachkomma_i))
                                / Power(10, nachkomma_i))
         Else error_b := True;
  Except error_b := True
  End
End;


{Procedure TStringArray.ZeileEinlesen(Zeile_i        : Interger;
                                     Trennzeichen_c : Char;
                                     was_s          : String);
Begin
End;      }


Function TStringArray.ZeileAuslesen_s(Zeile_i, von_i, bis_i : Integer;
                                      Trennzeichen_c        : Char) : String;

Var i : Integer;

Begin
  Result := '';
  If (bis_i   >=  0)    And (bis_i   <= Spaltenzahl_i) And
     (Zeile_i >=  0)    And (Zeile_i <= Zeilenzahl_i)  And
     (von_i   <= bis_i) And (von_i   >= 0)             Then
     Begin
       For i := von_i To Pred(bis_i) Do Result :=   Result
                                                  + Zellen_da_s[i, zeile_i]
                                                  + trennzeichen_c;
       Result := Result + zellen_da_s[bis_i, zeile_i]
     End
     Else error_b := True
End;


Procedure TStringArray.zeile_loeschen(Zeile_i : Integer);

Var s, z : Integer;

Begin
  If (Zeile_i >= 0) And (Zeile_i <= Zeilenzahl_i) Then
     Begin
       For z := zeile_i To zeilenzahl_i Do
           For s := 0 To spaltenzahl_i Do
               Zellen_da_s[s, z] := Zellen_da_s[s, Succ(z)];
       For s := 0 To spaltenzahl_i Do Zellen_da_s[s, zeilenzahl_i] := ''
     End
     Else error_b := True
End;


Function TStringArray.ZeilenFuellstatus_i(spalte_i : Integer) : Integer;
Begin
  Result := -1;
  If (Spalte_i > -1) And (Spalte_i <= Spaltenzahl_i) Then
     While Not(Result = zeilen_i_prop)                 And
           Not(StrRead_s(spalte_i, Succ(Result)) = '') Do
           Inc(Result)
     Else error_b := True
End;


Function TStringArray.SpaltenFuellstatus_i(Zeile_i : Integer) : Integer;
Begin
  Result := -1;
  If (Zeile_i > -1) And (Zeile_i <= Spaltenzahl_i) Then
     While Not(Result = zeilen_i_prop)                 And
           Not(StrRead_s(Succ(Result), Zeile_i) = '') Do
           Inc(Result)
     Else error_b := True
End;


Function TStringArray.voll_b(Spalte_i : Integer) : Boolean;
Begin
  If //ZeilenFuellstatus_i(Spalte_i) = zeilen_i_prop Then
     StrRead_s(Spalte_i, zeilen_i_Prop) = '' Then
     Result := False
     //Result := True
     Else //Result := False
          Result := True
End;


Procedure TStringArray.leeren;

Var s, z : Integer;

Begin
  For s := 0 To High(zellen_da_s) Do
      For z := 0 To High(zellen_da_s[0]) Do
          zellen_da_s[s, z] := ''
End;



end.
