unit StringGitter;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, Math;

type
  TBereich = Record //gesperrter Bereich
               Left_li, Top_li, Right_li, Bottom_li : LongInt;
             End;

  TStringGitter = class(TStringGrid)
  private
    { Private-Deklarationen }

    gesperrt_b  : Boolean;  //Ob ein Bereich und welcher Bereich
    g_bereich   : TBereich; //nicht zum Editieren freigegeben wird

    tz_c        : Char; //Trennzeichen

    nkstellen_i : Integer; //Nachkommastellen beim Import von Fliekommazahlen

    excresult_b : Boolean; //False, falls beim Umwandeln zwischen String und
                           //Zahlenformaten Fehler aufgetreten sind.
    Function fexcresult_b : Boolean; //Gibt excresult aus und setzt die Variable
                                     //auf True zurck.

  protected
    { Protected-Deklarationen }

    procedure SetEditText(ACol, ARow: Longint; Const Value: string); Override;

    Function SelectCell(ACol, ARow: Longint): Boolean; override;

  public
    { Public-Deklarationen }

    Constructor Create(AOwner : TComponent); Override;

    Function zeile_auslesen(row_i, von_i, bis_i : Integer) : String;

    Procedure FloatWrite(col_i, row_i : Integer;
                         zahl_e       : Extended);

    Procedure IntWrite(col_i, row_i : Integer;
                       zahl_i       : Integer);

    Procedure StrWrite(col_i, row_i : Integer;
                       string_s     : String);

    Function FloatRead_e(col_i, row_i : Integer) : Extended;

    Function IntRead_i(col_i, row_i : Integer) : Integer;

    Function StrRead_s(col_i, row_i : Integer) : String;

    Procedure DelRow(row_i : Integer);    //Zeile loeschen

    Procedure DelRowCut(row_i : Integer); //Zeile loeschen und Zeilenzahl runter

    Procedure leeren;

    Property exc_result_b : Boolean Read fexcresult_b Write excresult_b;

  published
    { Published-Deklarationen }

    Property Trennzeichen  : Char Read tz_c  Write tz_c;

    //Eigenschaften, die einen der Edition verwehrten Bereich definieren
    Property Sperren : Boolean Read gesperrt_b Write gesperrt_b;
    Property SperrbereichLeft   : LongInt Read  g_bereich.left_li
                                          Write g_bereich.left_li;
    Property SperrbereichTop    : LongInt Read  g_bereich.Top_li
                                          Write g_bereich.Top_li;
    Property SperrbereichRight  : LongInt Read  g_bereich.Right_li
                                          Write g_bereich.Right_li;
    Property SperrbereichBottom : LongInt Read  g_bereich.Bottom_li
                                          Write g_bereich.Bottom_li;

    Property Nachkommastellen : Integer Read nkstellen_i Write nkstellen_i;
  end;



procedure Register;



implementation



procedure Register;
begin
  RegisterComponents('Christoph', [TStringGitter]);
end;



Function TStringGitter.zeile_auslesen(row_i, von_i, bis_i : Integer) : String;

Var i : Integer;

Begin
  Result := '';
  For i := von_i To Pred(bis_i) Do Result := Result + Cells[i, row_i]
                                             + Trennzeichen;
  Result := Result + Cells[bis_i, row_i]
End;



Constructor TStringGitter.Create(AOwner : TComponent);
Begin
  Inherited Create(AOwner);

  Trennzeichen        := ';';

  gesperrt_b          := False;
  g_bereich.left_li   := 0;
  g_bereich.top_li    := 0;
  g_bereich.right_li  := 0;
  g_bereich.Bottom_li := 0;

  Nachkommastellen    := 3
End;



Function TStringGitter.SelectCell(ACol, ARow: Longint): Boolean;
Begin
  Result := Inherited SelectCell(ACol, ARow);
  If gesperrt_b Then
     With g_bereich Do
          If ((ACol < left_li) Or (ACol > right_li))  Or
             ((ARow < top_li)  Or (ARow > bottom_li)) Then
             Options := Options + [GoEditing{, GoAlwaysShowEditor}]
             Else Options := Options - [GoEditing{, GoAlwaysShowEditor}]
End;



procedure TStringGitter.SetEditText(ACol, ARow: Longint; Const Value: string);

Var puffer_s       : String;
    i              : Integer;
    trennzeichen_b : Boolean;

Begin
  trennzeichen_b := False;
  puffer_s       := Value;
  For i := 1 To Length(puffer_s) Do
      If puffer_s[i] = trennzeichen Then //Ein eventuell eingegebenes
         Begin                           //Trennzeichen wird abgefangen...
           trennzeichen_b := True;
           Delete(puffer_s, i, 1)
         End;
  If trennzeichen_b Then                 //...und auch aus dem String im
     With InPlaceEditor Do               //Inplaceeditor entfernt.
          Begin
            i        := SelStart;
            EditText := puffer_s;
            SelStart := Pred(i)
          End
     Else Inherited SetEditText(ACol, ARow, Value); //Nur wenn kein Trennzeichen
End;                                                //eingegeben wurde, wird das
                                                    //Ereignis "OnSetEditText"
                                                    //ausgelst.

Procedure TStringGitter.FloatWrite(col_i, row_i : Integer;
                                   zahl_e       : Extended);
Begin
  //Cells[col_i, row_i] := FloatToStrF(zahl_e, ffFixed, 18, 3);
  Cells[col_i, row_i] := FloatToStr(Round(zahl_e * Power(10, nachkommastellen))
                                    / Power(10, nachkommastellen));
End;



Procedure TStringGitter.IntWrite(col_i, row_i : Integer;
                                 zahl_i       : Integer);
Begin
  Cells[col_i, row_i] := IntToStr(zahl_i);
End;



Procedure TStringGitter.StrWrite(col_i, row_i : Integer;
                                     string_s     : String);
Begin
  Cells[col_i, row_i] := string_s;
End;



Function TStringGitter.StrRead_s(col_i, row_i : Integer) : String;
Begin
  Result := Cells[col_i, row_i];
End;



Function TStringGitter.FloatRead_e(col_i, row_i : Integer) : Extended;
Begin
  Try Result := StrToFloat(Cells[col_i, row_i]);
  Except exc_result_b := False;
         End
End;



Function TStringGitter.IntRead_i(col_i, row_i : Integer) : Integer;
Begin
  Try Result := StrToInt(Cells[col_i, row_i]);
  Except exc_result_b := False
         End
End;



Procedure TStringGitter.DelRow(row_i : Integer);

Var s, z : Integer;

Begin
  If Not(row_i = Pred(RowCount)) Then
     For z := row_i To Pred(RowCount) Do
         For s := 0 + FixedCols To Pred(ColCount) Do
             Cells[s, z] := Cells[s, Succ(z)];
  For s := 0 + FixedCols To Pred(ColCount) Do Cells[s, Pred(RowCount)] := ''
End;



Procedure TStringGitter.DelRowCut(row_i : Integer);
Begin
  DelRow(row_i);
  If RowCount > Succ(FixedRows) Then RowCount := Pred(RowCount)
End;



Function TStringGitter.fexcresult_b : Boolean;
Begin
  Result       := excresult_b;
  exc_result_b := True
End;



Procedure TStringGitter.leeren;

Var s, z : Integer;

Begin
  For z := 0 + FixedRows To Pred(RowCount) Do
      For s := 0 + FixedCols To Pred(ColCount) Do
          Cells[s, z] := ''
End;



end.
