unit wellenkalformular;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, Grids, NStringGridKomponente;

type
  tbanden = Array[1..60] Of Record
                              Pixel_i        : Integer;
                              wellenlaenge_e : Extended
                            End;

  teinzelccd = Record
                 banden     : tbanden;
                 anzahl_i   : Integer;
                 a_absch_e  : Extended;
                 steigung_e : Extended;
               End;

  tccd = Array[1..4] Of teinzelccd;

  TWellenKalForm = class(TForm)
    ok: TBitBtn;
    abbruch: TBitBtn;
    loeschen: TBitBtn;
    CCDZahlGroupBox: TGroupBox;
    DreiRadioButton: TRadioButton;
    VierRadioButton: TRadioButton;
    WellKalNStringGrid: TNStringGrid;
    procedure FormCreate(Sender: TObject);
    procedure loeschenClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure okClick(Sender: TObject);
  private
    { Private-Deklarationen }

    ccd          : tccd;   //Kalibrationsdaten der CCDs
    kalibriert_b : Boolean;//Wurde kalibriert?
    pfad_s       : String; //Pfad fr die Konfigurationsdatei

    Procedure laden;       //...der Einstellungen

    Procedure kalibrieren;//Durchfhrung der Kalibration der Wellenlngen
    
  public
    { Public-Deklarationen }

    Procedure uebernehmen(bez_s, pixel_s : String);//Aus dem Integrations-
                                                   //formular bernehmen

    Function KalLinOut_s(linie_i : Integer) : String;//bergibt die Kalibra-
                                                     //tionslinien zum Speichern
                                                     //in das Datenobjekt

    Function WaveLOut_e(pixel_i : Integer) : Extended;//Die Wellenlnge zum
                                              //angegebenen Pixel ausgeben
                                              //(zur graphischen Darstellung
                                              //in Datenobjekt)

    Procedure speichern;   //...der Einstellungen

    Function CCDZahl_i : Integer;
   
  end;

var
  WellenKalForm: TWellenKalForm;

implementation

{$R *.DFM}

procedure TWellenKalForm.FormCreate(Sender: TObject);
begin
  With WellkalNStringGrid Do
       Begin
         StrWrite(0,0, 'Bezeichnung');
         StrWrite(1,0, 'Pixel');
         StrWrite(2,0, 'Wellenlnge')
       End;
  GetDir(0, pfad_s);
  laden;
  kalibrieren
end;



Procedure TWellenKalForm.uebernehmen(bez_s, pixel_s : String);

Var wohin_i : Integer;

Begin
  //spiegeln;
  wohin_i := 0;
  With WellKalNStringGrid Do
       Begin
         While Not(StrRead_s(0, Succ(wohin_i)) = '') And
               Not(wohin_i = 60) Do
               Inc(wohin_i);
         Inc(wohin_i);
         If Not(wohin_i = 61) Then
            Begin
              StrWrite(0, wohin_i, bez_s);
              StrWrite(1, wohin_i, pixel_s)
            End
       End;
End;



Function TWellenKalForm.KalLinOut_s(linie_i : Integer) : String;
Begin
  Result := WellKalNStringGrid.ZeileAuslesen_s(linie_i, 0, 2);
End;



procedure TWellenKalForm.loeschenClick(Sender: TObject);
begin
  With WellKalNStringGrid Do ClearRow(row)
end;


Procedure TWellenKalForm.kalibrieren;

Var i         : Integer;
    puffer_i  : Integer;
    puffer_e  : Extended;

Function summexy_e(einzelccd : teinzelccd) : Extended; //Produktsumme aus
                                                       //Wellenlngen und Pixel
Var i : Integer;

Begin //summexy_e
  Result := 0;
  With einzelccd Do
       For i := 1 To anzahl_i Do
           Result := Result + banden[i].pixel_i * banden[i].wellenlaenge_e
End;

Function quadratsumme_e(einzelccd : teinzelccd) : Extended; //Quadratsumme der
                                                            //Pixel
Var i : Integer;

Begin  //quadratsumme
  Result := 0;
  With einzelccd Do
       For i := 1 To anzahl_i Do
           Result := Result + banden[i].pixel_i * banden[i].pixel_i
End;

Function p_summe_e(einzelccd : teinzelccd) : Extended; //Pixelsumme

Var i : Integer;

Begin
  Result := 0;
  With einzelccd Do
       For i := 1 To anzahl_i Do
           Result := Result + banden[i].pixel_i
End;

Function w_summe_e(einzelccd : teinzelccd) : Extended; //Summe der Wellenlngen

Var i : Integer;

Begin
  Result := 0;
  With einzelccd Do
       For i := 1 To anzahl_i Do
           Result := Result + banden[i].wellenlaenge_e
End;

Function regression_b(Var einzelccd : teinzelccd) : Boolean;

Var zaehler_e, nenner_e : Extended;

Begin
  Result := True;
  With einzelccd Do
       If anzahl_i > 1 Then
          Try zaehler_e  :=   anzahl_i * summexy_e(einzelccd)
                            - p_summe_e(einzelccd) * w_summe_e(einzelccd);
              nenner_e   :=   anzahl_i * (quadratsumme_e(einzelccd))
                            - Sqr(p_summe_e(einzelccd));
              steigung_e := zaehler_e / nenner_e;

              zaehler_e :=   w_summe_e(einzelccd)
                           - steigung_e * p_summe_e(einzelccd);
              nenner_e  := anzahl_i;
              a_absch_e := zaehler_e / nenner_e;
          Except Result := False
                 End
       Else Result := False
End;

Procedure drauflegen(Var einzelccd : teinzelccd;
                         puffer_i  : Integer;
                         puffer_e  : Extended);
Begin  //drauflegen
  With einzelccd Do
       If anzahl_i < 60 Then
          Begin
            Inc(anzahl_i);
            banden[anzahl_i].pixel_i        := puffer_i;
            banden[anzahl_i].wellenlaenge_e := puffer_e
          End
End;

begin //kalibrieren
  For i := 1 To 4 Do ccd[i].anzahl_i := 0;
  i := 1;
  With WellKalNStringGrid Do
       While Not(i = 61) And Not(StrRead_s(0, i) = '') Do
           Begin
             ExecStatus_b_Prop;
             puffer_i := IntRead_i(1, i);
             puffer_e := FloatRead_e(2, i);
             If ExecStatus_b_Prop Then
                If puffer_i < 2100 Then
                   drauflegen(ccd[1], puffer_i, puffer_e)
                   Else If puffer_i < 4200 Then
                           drauflegen(ccd[2], puffer_i, puffer_e)
                           Else If puffer_i < 6300 Then
                                   drauflegen(ccd[3], puffer_i, puffer_e)
                                   Else If puffer_i < 8400 Then
                                           drauflegen(ccd[4],puffer_i,puffer_e);
             Inc(i)
           End;

  kalibriert_b := True;
  For i := 1 To CCDZahl_i Do If Not(regression_b(ccd[i])) Then kalibriert_b := False;
end;



Function TWellenKalForm.WaveLOut_e(pixel_i : Integer) : Extended;
Begin
  If kalibriert_b Then
     Begin
       If pixel_i < 2100 Then
          Result := ccd[1].steigung_e * pixel_i + ccd[1].a_absch_e
          Else
          If pixel_i < 4200 Then
             Result := ccd[2].steigung_e * pixel_i + ccd[2].a_absch_e
             Else
             If pixel_i < 6300 Then
                Result := ccd[3].steigung_e * pixel_i + ccd[3].a_absch_e
                Else
                If pixel_i < 8400 Then
                   Result := ccd[4].steigung_e * pixel_i + ccd[4].a_absch_e
     End
     Else Result := pixel_i
End;



Procedure TWellenKalForm.laden;

Var datei_txt : TextFile;
    fehler_b  : Boolean;
    i,j       : Integer;
    puffer_s  : String;

Begin
  Try AssignFile(datei_txt, pfad_s + '\wellenlaengen.spg');
      Reset(datei_txt);
      fehler_b := False;
  Except fehler_b := True;
  End;
  If Not(fehler_b) Then
     Begin
       Try With WellKalNStringGrid Do
                For j := 0 To 2 Do
                    For i := 1 To 60 Do
                        Begin
                          ReadLn(datei_txt, puffer_s);
                          StrWrite(j, i, puffer_s)
                        End;
           ReadLn(datei_txt, puffer_s);
           If puffer_s = '3 CCDs' Then
              Begin
                DreiRadioButton.Checked := True;
                VierRadioButton.Checked := False
              End
              Else
              Begin
                DreiRadioButton.Checked := False;
                VierRadioButton.Checked := True;
              End;
       Except fehler_b := True;
       End;
       Try CloseFile(datei_txt);
       Except fehler_b := True;
       End
     End;
  If fehler_b Then
     Application.MessageBox('Fehler beim Laden der Wellenlngenkalibration!',
                            'Fehlermeldung', Mb_Ok + MB_ICONSTOP)
End;



procedure TWellenKalForm.FormShow(Sender: TObject);
begin
  laden;
end;


Procedure TWellenKalForm.speichern;

Var datei_txt : TextFile;
    fehler_b  : Boolean;
    i, j      : Integer;

Begin
  Try AssignFile(datei_txt, pfad_s + '\wellenlaengen.spg');
      Rewrite(datei_txt);
      fehler_b := False;
  Except fehler_b := True;
  End;
  If Not(fehler_b) Then
     Begin
       Try With WellKalNStringGrid Do
                For j := 0 To 2 Do
                    For i := 1 To 60 Do WriteLn(datei_txt, StrRead_s(j,i));
           If DreiRadioButton.Checked Then WriteLn(datei_txt, '3 CCDs')
              Else WriteLn(datei_txt, '4 CCDs');
       Except fehler_b := True;
       End;
       Try CloseFile(datei_txt);
       Except fehler_b := True
       End
     End;
  If fehler_b Then
    Application.MessageBox('Fehler beim Speichern der Wellenlngenkalibration!',
                           'Fehlermeldung', Mb_Ok + MB_ICONSTOP);
end;



procedure TWellenKalForm.okClick(Sender: TObject);
begin
  speichern;
  kalibrieren
end;



Function TWellenKalForm.CCDZahl_i;
Begin
  If DreiRadioButton.Checked Then Result := 3
     Else Result := 4
End;



end.
